/**
 * @file main.c
 *
 * Basic Keil uVision project
 *
 * @author Tecnologix, 2017
 */
#include "stm32f4xx_hal.h"              // Keil::Device:STM32Cube HAL:Common
#include "gpio.h"
#include "debug.h"
#include "EventRecorder.h"
#include <stdio.h>
#include <stdbool.h>

/* =======================================================================
 * MACROS
 * ======================================================================= */

#define LED2_GPIO		(GPIOA) 
#define	LED2_PIN		(5)

/* =======================================================================
 * EXTERN SYMBOLS
 * ======================================================================= */

extern uint32_t uwTick;

/* =======================================================================
 * GLOBAL VARIABLES
 * ======================================================================= */

/* Just a modulo 2 counter */
uint32_t modulo_2 = 0;

/* Blink delay */
uint32_t delay = 1000;

bool dbg_cmd_out = false;

/* =======================================================================
 * PRIVATE FUNCTIONS
 * ======================================================================= */

/*
 * Initializes leds
 */
static void init_leds(void)
{
	/* Clock GPIOA peripheral */
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN;
	
	/* Set LED2 port and pin */
	gpio_set_pin(LED2_GPIO,LED2_PIN, TLX_GPIO_MODE_GPOUT, TLX_GPIO_SPEED_50MHZ, 0);
}



/* =======================================================================
 * PUBLIC FUNCTIONS
 * ======================================================================= */
 
int main(void)
{		
	HAL_Init();
	
	dbg_intr = false;
	
	EventRecorderInitialize (EventRecordAll, 1);
	
	NVIC_EnableIRQ(EXTI0_IRQn);
	
	init_leds();
	
	gpio_set(LED2_GPIO, LED2_PIN);
	
	while(1)
	{
		modulo_2 = (modulo_2 + 1) & 1;
		
		if (modulo_2)
		{
			delay = 500;
		}
		else 
		{
			delay = 1000;
		}
		
		dbg_led_on = true;
		dbg_led_on_cnt = (dbg_led_on_cnt + 1)%20;
		printf("LED ON \n");
		
		EventRecord2(EventLevelOp | 0x01, (uint32_t)dbg_led_on, (uint32_t)!dbg_led_on);
		
		gpio_set(LED2_GPIO, LED2_PIN);
			
		HAL_Delay(delay);
		
		dbg_led_on = false;
		printf("LED OFF \n");
		gpio_clr(LED2_GPIO, LED2_PIN);
		
		EventRecord2(EventLevelOp | 0x01, (uint32_t)dbg_led_on, (uint32_t)!dbg_led_on);
		
		HAL_Delay(delay);
		
		if (dbg_trigger_interrupt_when_ready == 1)
		{
			NVIC_SetPendingIRQ(EXTI0_IRQn);
			
			dbg_trigger_interrupt_when_ready = 0;
		}
		
		/* Just a try */
		delay = 0;
		
		__nop();
		
		__nop();
	}
}


/* =======================================================================
 * INTERRUPT HANDLER
 * ======================================================================= */

/* Interrupt handler */
void SysTick_Handler(void)
{
	uwTick++;
}

void EXTI0_IRQHandler(void)
{
	uint8_t i;
	
	NVIC_ClearPendingIRQ(EXTI0_IRQn);
	
	dbg_intr = true;
	
	for(i=0; i<20; i++)
	{
		__nop();
	}
	
	dbg_intr = false;
}
